/*
 * main.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
/*
** Example Winamp .RAW input plug-in
** Copyright (c) 1998, Justin Frankel/Nullsoft Inc.
**
** benski - Cinco de Mayo 2008
**  Updated for Winamp 5.5 const-correct In_Module definition 
**
*/

//#define ORIGINAL
#if defined (ORIGINAL) // {
#include <windows.h>
#include <tchar.h>

#include <Winamp/in2.h>
#else // } {
#include "plugin.h"
#endif // }

// avoid CRT. Evil. Big. Bloated. Only uncomment this code if you are using 
// 'ignore default libraries' in VC++. Keeps DLL size way down.
// /*
BOOL WINAPI _DllMainCRTStartup(HANDLE hInst, ULONG ul_reason_for_call, LPVOID lpReserved)
{
	return TRUE;
}
// */

// post this to the main window at end of file (after playback as stopped)
#define WM_WA_MPEG_EOF WM_USER+2


#if defined (ORIGINAL) // {
// raw configuration.
#define NCH 2
#define SAMPLERATE 44100
#define BPS 16
#else // } {
#define RUN_LATE
//#define MOD_LATE
#define NCH (p.dec.a.out.nch)
#define SAMPLERATE (p.dec.a.out.rate)
#define BPS (p.dec.a.out.bps)
#endif // }


In_Module mod;			// the output module (filled in near the bottom of this file)

#if defined (ORIGINAL) // {
in_char lastfn[MAX_PATH];	// currently playing file (used for getting info on the current file)

int file_length;		// file length, in bytes
#else // } {
plugin_t p;
char display[1024];
char plugin[1024];
#endif // }
#if defined (ORIGINAL) // {
int decode_pos_ms;		// current decoding position, in milliseconds. 
						// Used for correcting DSP plug-in pitch changes
#else
#define	decode_pos_ms (p.dec.a.s.time)
#define	sample_buffer (p.dec.a.s.rep)
#endif // }
int paused;				// are we paused?
volatile int seek_needed; // if != -1, it is the point that the decode 
						  // thread should seek to, in ms.

HANDLE input_file=INVALID_HANDLE_VALUE; // input file handle

volatile int killDecodeThread=0;			// the kill switch for the decode thread
HANDLE thread_handle=INVALID_HANDLE_VALUE;	// the handle to the decode thread


DWORD WINAPI DecodeThread(LPVOID b); // the decode thread procedure

void config(HWND hwndParent)
{
#if defined (ORIGINAL) // {
	MessageBox(hwndParent,
		_T("No configuration. .RAW files must be 44/16/2"),
		_T("Configuration"),MB_OK);
#else // } {
	prefs_edit(&p.ps.play,&p.ps.conv,hwndParent);
#endif // }
	// if we had a configuration box we'd want to write it here (using DialogBox, etc)
}
void about(HWND hwndParent)
{
#if defined (ORIGINAL) // {
	MessageBox(hwndParent,_T("Nullsoft RAW Player, by Justin Frankel"),
		_T("About Nullsoft RAW Player"),MB_OK);
#else // } {
	MessageBox(hwndParent,_T(CAPTION ", by Peter Belkner."),
		_T("About " CAPTION),MB_OK);
#endif // }
}

void init() { 
	/* any one-time initialization goes here (configuration reading, etc) */ 
#if ! defined (ORIGINAL) // {
  pb_str_t *root=NULL;
  pb_str_t *home=NULL;
  
#if defined (DEBUG) // {
  AllocConsole();
  freopen("conin$", "r", stdin);
  freopen("conout$", "w", stdout);
  freopen("conout$", "w", stderr);
#endif // }

  if (NULL==(root=pb_str_home(1)))
    goto cleanup;

  if (NULL==(home=pb_str_cat_ps(root,PS_SEP,_PS("Plugins\\in_ffsox"),NULL)))
    goto cleanup;

  if (0!=ffsox_load_sox(home))
    goto cleanup;

  if (0!=ffsox_load_ffmpeg(home))
    goto cleanup;

  buf_init(&p.ps.play.exts.display,display,sizeof display);
  buf_init(&p.ps.play.exts.plugin,plugin,sizeof plugin);

  if (NULL==plugin_init(&p,&mod,root,home))
    goto cleanup;

  mod.UsesOutputPlug|=IN_MODULE_FLAG_REPLAYGAIN;
  mod.UsesOutputPlug|=IN_MODULE_FLAG_REPLAYGAIN_PREAMP;
cleanup:
#if 0 // {
  if (NULL!=home)
    free(home);

  if (NULL!=root)
    free(root);
#else // } {
  return;
#endif // }
#endif // }
}

void quit() { 
	/* one-time deinit, such as memory freeing */ 
#if ! defined (ORIGINAL) // {
  plugin_cleanup(&p);
#endif // }
}

int isourfile(const in_char *fn) { 
// used for detecting URL streams.. unused here. 
// return !strncmp(fn,"http://",7); to detect HTTP streams, etc
	return 0; 
} 

#if ! defined (ORIGINAL) // {
int open_mod()
{
	int maxlatency;

	// -1 and -1 are to specify buffer and prebuffer lengths.
	// -1 means to use the default, which all input plug-ins should
	// really do.
	maxlatency = mod.outMod->Open(SAMPLERATE,NCH,BPS, -1,-1); 

	// maxlatency is the maxium latency between a outMod->Write() call and
	// when you hear those samples. In ms. Used primarily by the visualization
	// system.

	if (maxlatency < 0) // error opening device
	{
#if defined (ORIGINAL) // {
		CloseHandle(input_file);
		input_file=INVALID_HANDLE_VALUE;
#endif // }
		return 1;
	}
	// dividing by 1000 for the first parameter of setinfo makes it
	// display 'H'... for hundred.. i.e. 14H Kbps.
	mod.SetInfo((SAMPLERATE*BPS*NCH)/1000,SAMPLERATE/1000,NCH,1);

	// initialize visualization stuff
	mod.SAVSAInit(maxlatency,SAMPLERATE);
	mod.VSASetInfo(SAMPLERATE,NCH);

	// set the output plug-ins default volume.
	// volume is 0-255, -666 is a token for
	// current volume.
	mod.outMod->SetVolume(-666); 

	return 0;
}

void close_mod()
{
	for (;;) {
		mod.outMod->CanWrite();		// some output drivers need CanWrite
							    	// to be called on a regular basis.

		if (!mod.outMod->IsPlaying()) 
			break;

		Sleep(10);
	}

	// close output system
	mod.outMod->Close();

	// deinitialize visualization
	mod.SAVSADeInit();
}
#endif // }

// called when winamp wants to play a file
int play(const in_char *fn) 
{ 
#if defined (ORIGINAL)
	int maxlatency;
#endif
	DWORD thread_id;

	paused=0;
	decode_pos_ms=0;
	seek_needed=-1;

	
	
	// CHANGEME! Write your own file opening code here
#if defined (ORIGINAL) // {
	input_file = CreateFile(fn,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,
		OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
	if (input_file == INVALID_HANDLE_VALUE) // error opening file
	{
		// we return error. 1 means to keep going in the playlist, -1
		// means to stop the playlist.
		return 1;
	}

	file_length=GetFileSize(input_file,NULL);


	
	_tcscpy(lastfn,fn);

	// -1 and -1 are to specify buffer and prebuffer lengths.
	// -1 means to use the default, which all input plug-ins should
	// really do.
	maxlatency = mod.outMod->Open(SAMPLERATE,NCH,BPS, -1,-1); 

	// maxlatency is the maxium latency between a outMod->Write() call and
	// when you hear those samples. In ms. Used primarily by the visualization
	// system.

	if (maxlatency < 0) // error opening device
	{
		CloseHandle(input_file);
		input_file=INVALID_HANDLE_VALUE;
		return 1;
	}
	// dividing by 1000 for the first parameter of setinfo makes it
	// display 'H'... for hundred.. i.e. 14H Kbps.
	mod.SetInfo((SAMPLERATE*BPS*NCH)/1000,SAMPLERATE/1000,NCH,1);

	// initialize visualization stuff
	mod.SAVSAInit(maxlatency,SAMPLERATE);
	mod.VSASetInfo(SAMPLERATE,NCH);

	// set the output plug-ins default volume.
	// volume is 0-255, -666 is a token for
	// current volume.
	mod.outMod->SetVolume(-666); 
#else // } {
#if defined (DEBUG) // {
	fwprintf(stderr,L">>> START: \"%s\"\n",fn);
#endif // }
	if (NULL==plugin_start(&p,fn))
		goto error_start;

#if ! defined (RUN_LATE) // {
	if (NULL==plugin_run(&p))
		goto error_run;
#endif // }
#if ! defined (MOD_LATE) // {
	if (0!=open_mod())
		goto error_mod;
#endif // }
#endif // }

	// launch decode thread
	killDecodeThread=0;
	thread_handle = (HANDLE) 
		CreateThread(NULL,0,(LPTHREAD_START_ROUTINE) DecodeThread,NULL,0,&thread_id);
	
#if ! defined (ORIGINAL) // {
	if (NULL==thread_handle)
		goto error_thread;
#endif // }

	return 0; 
#if ! defined (ORIGINAL) // {
error_thread:
#if ! defined (MOD_LATE) // {
    close_mod();
error_mod:
#endif // }
#if ! defined (RUN_LATE) // {
	plugin_kill(&p);
	plugin_wait(&p);
error_run:
#endif // }
	plugin_stop(&p,0);
error_start:
#if defined (DEBUG) // {
	fwprintf(stderr,L"### FAIL: \"%s\"\n",fn);
#endif // }
	return 1; 
#endif // }
}

// standard pause implementation
#if defined (ORIGINAL) // {
void pause() { paused=1; mod.outMod->Pause(1); }
void unpause() { paused=0; mod.outMod->Pause(0); }
#else // } {
void pause()
{
	paused=1;
	mod.outMod->Pause(1);
	plugin_pause(&p);
}

void unpause()
{
	plugin_unpause(&p);
	mod.outMod->Pause(0);
	paused=0;
}
#endif // }
int ispaused() { return paused; }


// stop playing.
void stop() { 
#if ! defined (ORIGINAL) // {
	int killed=0;

#if ! defined (MOD_LATE) // {
	plugin_kill(&p);
#endif // }
#endif // }

	if (thread_handle != INVALID_HANDLE_VALUE)
	{
#if defined (ORIGINAL) // {
		killDecodeThread=1;
#else // } {
		if (!killDecodeThread) {
			killDecodeThread=1;
			killed=1;
		}
#endif // }
		if (WaitForSingleObject(thread_handle,10000) == WAIT_TIMEOUT)
		{
			MessageBox(mod.hMainWindow,_T("error asking thread to die!\n"),
				_T("error killing decode thread"),0);
			TerminateThread(thread_handle,0);
		}
		CloseHandle(thread_handle);
		thread_handle = INVALID_HANDLE_VALUE;
	}

#if defined (ORIGINAL) // {
	// close output system
	mod.outMod->Close();

	// deinitialize visualization
	mod.SAVSADeInit();
	

	// CHANGEME! Write your own file closing code here
	if (input_file != INVALID_HANDLE_VALUE)
	{
		CloseHandle(input_file);
		input_file=INVALID_HANDLE_VALUE;
	}
#else // } {
#if ! defined (MOD_LATE) // {
	close_mod();
#endif // }
#if ! defined (RUN_LATE) // {
	plugin_wait(&p);
#endif // }
	plugin_stop(&p,killed);
#endif // }
}


// returns length of playing track
int getlength() {
#if defined (ORIGINAL) // {
	return (file_length*10)/(SAMPLERATE/100*NCH*(BPS/8)); 
#else // } {
#if defined (PLAY) // {
	return track_get_length(&p.play);
#else // } {
	return track_get_length(&p.dec.t);
#endif // }
#endif // }
}


// returns current output position, in ms.
// you could just use return mod.outMod->GetOutputTime(),
// but the dsp plug-ins that do tempo changing tend to make
// that wrong.
int getoutputtime() { 
	return decode_pos_ms+
		(mod.outMod->GetOutputTime()-mod.outMod->GetWrittenTime()); 
}


// called when the user releases the seek scroll bar.
// usually we use it to set seek_needed to the seek
// point (seek_needed is -1 when no seek is needed)
// and the decode thread checks seek_needed.
void setoutputtime(int time_in_ms) { 
	seek_needed=time_in_ms; 
}


// standard volume/pan functions
void setvolume(int volume) { mod.outMod->SetVolume(volume); }
void setpan(int pan) { mod.outMod->SetPan(pan); }

// this gets called when the use hits Alt+3 to get the file info.
// if you need more info, ask me :)

int infoDlg(const in_char *fn, HWND hwnd)
{
	// CHANGEME! Write your own info dialog code here
	return 0;
}


#if ! defined (ORIGINAL) // {
// return 1 if you want winamp to show it's own file info dialogue, 0 if you want to show your own (via In_Module.InfoBox)
// if returning 1, remember to implement winampGetExtendedFileInfo("formatinformation")!
__declspec(dllexport) int winampUseUnifiedFileInfoDlg(const wchar_t * fn)
{
    return 1;
}
#endif // }

// this is an odd function. it is used to get the title and/or
// length of a track.
// if filename is either NULL or of length 0, it means you should
// return the info of lastfn. Otherwise, return the information
// for the file in filename.
// if title is NULL, no title is copied into it.
// if length_in_ms is NULL, no length is copied into it.
void getfileinfo(const in_char *filename, in_char *title, int *length_in_ms)
{
#if defined (ORIGINAL) // {
	if (!filename || !*filename)  // currently playing file
	{
		if (length_in_ms) *length_in_ms=getlength();
		if (title) // get non-path portion.of filename
		{
			in_char *p=lastfn+_tcslen(lastfn);
			while (*p != '\\' && p >= lastfn) p--;
			_tcscpy(title,++p);
		}
	}
	else // some other file
	{
		if (length_in_ms) // calculate length
		{
			HANDLE hFile;
			hFile = CreateFile(filename,GENERIC_READ,FILE_SHARE_READ|FILE_SHARE_WRITE,NULL,
				OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);
			if (hFile != INVALID_HANDLE_VALUE)
			{
				*length_in_ms = (GetFileSize(hFile,NULL)*10)/(SAMPLERATE/100*NCH*(BPS/8));
				CloseHandle(hFile);
			}
			else *length_in_ms=-1000; // the default is unknown file length (-1000).
		}
		if (title) // get non path portion of filename
		{
			const in_char *p=filename+_tcslen(filename);
			while (*p != '\\' && p >= filename) p--;
			_tcscpy(title,++p);
		}
	}
#else // } {
	track_t *t;

	if (filename && *filename)  // not currently playing file
		t=track_reopen(&p.list,filename,&p.ps.play,-1);
#if defined (PLAY) // {
	else if (NULL!=p.play.ffi.ic)
		t=&p.play;
#else // } {
	else if (NULL!=p.dec.t.ffi.ic)
		t=&p.dec.t;
#endif // }
    else
		t=NULL;

    if (NULL!=t)
		track_getfileinfo(t,title,length_in_ms);
#endif // }
}

#if ! defined (ORIGINAL) // {
__declspec (dllexport) int winampGetExtendedFileInfoW (wchar_t *filename,
    char *metadata, wchar_t *ret, int retlen)
{
	track_t *t;

	if (filename && *filename)  // not currently playing file
		t=track_reopen(&p.list,filename,&p.ps.play,1);
#if defined (PLAY) // {
	else if (NULL!=p.play.ffi.ic)
		t=&p.play;
#else // } {
	else if (NULL!=p.dec.t.ffi.ic)
		t=&p.dec.t;
#endif // }
    else
		t=NULL;

	return NULL==t||NULL==track_getfileinfo_ext(t,metadata,ret,retlen)?0:1;
}
#endif // }

void eq_set(int on, char data[10], int preamp) 
{ 
	// most plug-ins can't even do an EQ anyhow.. I'm working on writing
	// a generic PCM EQ, but it looks like it'll be a little too CPU 
	// consuming to be useful :)
	// if you _CAN_ do EQ with your format, each data byte is 0-63 (+20db <-> -20db)
	// and preamp is the same. 
}


// render 576 samples into buf. 
// this function is only used by DecodeThread. 

// note that if you adjust the size of sample_buffer, for say, 1024
// sample blocks, it will still work, but some of the visualization 
// might not look as good as it could. Stick with 576 sample blocks
// if you can, and have an additional auxiliary (overflow) buffer if 
// necessary.. 
#if defined (ORIGINAL) // {
int get_576_samples(char *buf)
{
	DWORD l;
	// CHANGEME! Write your own sample getting code here
	ReadFile(input_file,buf,576*NCH*(BPS/8),&l,NULL);
	return l;
}
#else // } {
int get_576_samples(char *buf)
{
	samples_t *s=&p.dec.a.s;

    WaitForSingleObject(s->hMutex,INFINITE);

	while (STATE_RUN==s->state&&s->wp<s->mp) {
		SignalObjectAndWait(s->hMutex,s->hEventRead,INFINITE,FALSE);
		WaitForSingleObject(s->hMutex,INFINITE);
	}

	return s->wp-s->rep;
}

void done_576_samples()
{
	samples_t *s=&p.dec.a.s;

	// reset the sample buffer.
	s->wp=s->rep;

	// signal the writer thread.
   	SetEvent(s->hEventWrite);
    ReleaseMutex(s->hMutex);
}
#endif // }



DWORD WINAPI DecodeThread(LPVOID b)
{
	int done=0; // set to TRUE if decoding has finished
#if ! defined (ORIGINAL) // {
#if defined (RUN_LATE) // {
	if (NULL==plugin_run(&p))
		goto cleanup_run;
#endif // }
#if defined (MOD_LATE) // {
	if (0!=open_mod())
		goto cleanup_mod;
#endif // }
#endif // }
	while (!killDecodeThread) 
	{
		if (seek_needed != -1) // seek is needed.
		{
#if defined (ORIGINAL) // {
			int offs;
			decode_pos_ms = seek_needed;
			seek_needed=-1;
			done=0;
			mod.outMod->Flush(decode_pos_ms); // flush output device and set 
											  // output position to the seek position
			offs = MulDiv(decode_pos_ms,SAMPLERATE,1000); // decode_pos_ms*SAMPLERATE/1000
			SetFilePointer(input_file,offs*NCH*(BPS/8),NULL,FILE_BEGIN); // seek!
#else // } {
			if (!killDecodeThread&&0==plugin_seek(&p,seek_needed)) {
			  mod.outMod->Flush(seek_needed); // flush output device and set 
											  // output position to the seek position
			  done=0;
			}

			seek_needed=-1;
#endif // }
		}

		if (done) // done was set to TRUE during decoding, signaling eof
		{
#if defined (ORIGINAL) // {
			mod.outMod->CanWrite();		// some output drivers need CanWrite
									    // to be called on a regular basis.

			if (!mod.outMod->IsPlaying()) 
			{
				// we're done playing, so tell Winamp and quit the thread.
#if defined (ORIGINAL) // {
				PostMessage(mod.hMainWindow,WM_WA_MPEG_EOF,0,0);
				return 0;	// quit thread
#else // } {
				goto cleanup;	// quit thread
#endif // }
			}
			Sleep(10);		// give a little CPU time back to the system.
#else // } {
			goto cleanup;	// quit thread
#endif // }
		}
		else if (mod.outMod->CanWrite() >= ((576*NCH*(BPS/8))*(mod.dsp_isactive()?2:1)))
			// CanWrite() returns the number of bytes you can write, so we check that
			// to the block size. the reason we multiply the block size by two if 
			// mod.dsp_isactive() is that DSP plug-ins can change it by up to a 
			// factor of two (for tempo adjustment).
		{	
			int l=576*NCH*(BPS/8);			       // block length in bytes
#if defined (ORIGINAL) // {
			static char sample_buffer[576*NCH*(BPS/8)*2]; 
												   // sample buffer. twice as 
												   // big as the blocksize
#endif // }

			l=get_576_samples(sample_buffer);	   // retrieve samples

			if (!l)			// no samples means we're at eof
			{
				done=1;
			}
			else	// we got samples!
			{
				// give the samples to the vis subsystems
				mod.SAAddPCMData((char *)sample_buffer,NCH,BPS,decode_pos_ms);	
				mod.VSAAddPCMData((char *)sample_buffer,NCH,BPS,decode_pos_ms);
				// adjust decode position variable
#if defined (ORIGINAL) // {
				decode_pos_ms+=(576*1000)/SAMPLERATE;	
#else // } {
				decode_pos_ms+=(576.0*1000)/SAMPLERATE;	
#endif // }

				// if we have a DSP plug-in, then call it on our samples
				if (mod.dsp_isactive()) 
					l=mod.dsp_dosamples(
						(short *)sample_buffer,l/NCH/(BPS/8),BPS,NCH,SAMPLERATE
					  ) // dsp_dosamples
					  *(NCH*(BPS/8));

				// write the pcm data to the output system
				mod.outMod->Write(sample_buffer,l);
			}
#if ! defined (ORIGINAL) // {
			done_576_samples();
#endif // }
		}
		else Sleep(20); 
		// if we can't write data, wait a little bit. Otherwise, continue 
		// through the loop writing more data (without sleeping)
	}
#if ! defined (ORIGINAL) // {
cleanup:
#if defined (MOD_LATE) // {
	close_mod();
cleanup_mod:
#endif // }
#if defined (RUN_LATE) // {
	plugin_kill(&p);
	plugin_wait(&p);
cleanup_run:
#endif // }
	if (!killDecodeThread) {
		killDecodeThread=1;
		PostMessage(mod.hMainWindow,WM_WA_MPEG_EOF,0,0);
	}
#endif // }
	return 0;
}


// module definition.

In_Module mod = 
{
	IN_VER,	// defined in IN2.H
#if defined (ORIGINAL) // {
	"Nullsoft RAW Player v0.0 "
#else // } {
	CAPTION " "
#endif // }
	// winamp runs on both alpha systems and x86 ones. :)
#ifdef __alpha
	"(AXP)"
#else
	"(x86)"
#endif
	,
	0,	// hMainWindow (filled in by winamp)
	0,  // hDllInstance (filled in by winamp)
#if defined (ORIGINAL) // {
	"RAW\0RAW Audio File (*.RAW)\0"
#elif defined (STATIC_EXTS) // } {
	"FLAC\0FLAC Audio File (*.FLAC)\0"
	"MP3\0MP3 Audio File (*.MP3)\0"
	"WV\0WV Audio File (*.WV)\0"
	"WAV\0WAV Audio File (*.WAV)\0"
	"M2TS\0M2TS Video File (*.M2TS)\0"
	"MKV\0MKV Video File (*.MKV)\0"
	"MP4\0MP4 Video File (*.MP4)\0"
	"FLV\0FLV Video File (*.FLV)\0"
	"WMV\0WMV Video File (*.WMV)\0"
	"VOB\0VOB Video File (*.VOB)\0"
	"AVI\0AVI Video File (*.AVI)\0"
#else // } {
	plugin
#endif // }
	// this is a double-null limited list. "EXT\0Description\0EXT\0Description\0" etc.
	,
#if defined (ORIGINAL) // {
	1,	// is_seekable
#else // } {
	1,	// is_seekable
#endif // }
	1,	// uses output plug-in system
	config,
	about,
	init,
	quit,
	getfileinfo,
	infoDlg,
	isourfile,
	play,
	pause,
	unpause,
	ispaused,
	stop,
	
	getlength,
	getoutputtime,
	setoutputtime,

	setvolume,
	setpan,

	0,0,0,0,0,0,0,0,0, // visualization calls filled in by winamp

	0,0, // dsp calls filled in by winamp

	eq_set,

	NULL,		// setinfo call filled in by winamp

	0 // out_mod filled in by winamp

};

// exported symbol. Returns output module.

__declspec( dllexport ) In_Module * winampGetInModule2()
{
	return &mod;
}

#define EXTENDED_READ
#if ! defined (ORIGINAL) && defined (EXTENDED_READ) // {
/*
 * http://forums.winamp.com/showthread.php?postid=2062773#post2062773
 */

/*
 * Open the file and prepare to decode.
 * parametes:
 * const char *fn - the file to decode
 * int *size - set *size to the total number of bytes that this file will decode to. You can set this to -1 if you don't know, but some things won't work. (like percentage estimates in the transcoder)
 * int *bps - set this to the number of bits per sample in the output
 * int *nch - set this to the number of channels in the output
 * int *srate - set this to the sample rate (i.e, 44100)
 * 
 * Note: try and use the values already in bps, nch and srate.
 * 
 * return value: a handle which will be passed back you in calls to getData and close. Return 0 on any error (such as the file not existing).
 */
__declspec( dllexport ) intptr_t winampGetExtendedRead_open_float(const char *fn, int *size,
    int *bps, int *nch, int *srate)
{
#if defined (DEBUG)
  fprintf(stderr,">>> open_float - \"%s\"\n",fn);
  fprintf(stderr,">>> open_float - bps: %d, nch: %d, srate: %d\n",*bps,*nch,*srate);
#endif
  return 0;
}

__declspec( dllexport ) intptr_t winampGetExtendedRead_open(const char *fn, int *size,
    int *bps, int *nch, int *srate)
{
#if defined (DEBUG)
  fprintf(stderr,">>> open - \"%s\"\n",fn);
  fprintf(stderr,">>> open - bps: %d, nch: %d, srate: %d\n",*bps,*nch,*srate);
#endif
  return 0;
}

__declspec( dllexport ) intptr_t winampGetExtendedRead_openW(const wchar_t *fn, int *size,
    int *bps, int *nch, int *srate)
{
  decoder_t *dec;
  double length;

#if defined (DEBUG)
  fwprintf(stderr,L">>> openW - \"%s\"\n",fn);
  fwprintf(stderr,L">>> openW - bps: %d, nch: %d, srate: %d\n",*bps,*nch,*srate);
#endif
  if (NULL==(dec=calloc(1,sizeof *dec)))
    goto calloc_error;

  if (NULL==decoder_init(dec,&p.ps.conv,NULL))
    goto init_error;

  if (NULL==decoder_start(dec,fn,4096))
    goto start_error;

  *bps=dec->a.out.bps;
  *nch=dec->a.out.nch;
  *srate=dec->a.out.rate;
  length=track_get_length(&dec->t);
  *size=round(0.001*length*(*bps/8)*(*nch)*(*srate));
#if defined (DEBUG)
  fwprintf(stderr,L"<<< openW - bps: %d, nch: %d, srate: %d\n",*bps,*nch,*srate);
#endif

  if (NULL==decoder_run(dec))
    goto run_error;

  return (intptr_t)dec;
run_error:
  decoder_stop(dec,NULL,NULL);
start_error:
  decoder_cleanup(dec);
init_error:
  free(dec);
calloc_error:
  return 0;
}

__declspec( dllexport ) intptr_t winampGetExtendedRead_openW_float(const wchar_t *fn, int *size,
    int *bps, int *nch, int *srate)
{
#if defined (DEBUG)
  fwprintf(stderr,L">>> openW_float - \"%s\"\n",fn);
  fprintf(stderr,">>> openW_float - bps: %d, nch: %d, srate: %d\n",*bps,*nch,*srate);
#endif
  return 0;
}

/*
 * Decode some of the file. This will be called multiple times until either the whole file is decoded or the decoding is cancelled.
 * parameters:
 * intptr_t handle: the handle you returned on winampGetExtendedRead_open
 * char *dest: a buffer to write the decoded file to
 * int len: the size of the buffer dest
 * int *killswitch: check this periodically. If it is not zero, return as soon as possible, the decoding has been cancelled.
 * 
 * return value: the number of bytes in dest. If this is zero, then winamp will assume that whole file is decoded.
 */
__declspec( dllexport ) intptr_t winampGetExtendedRead_getData(intptr_t handle,
    char *dest, int len, int *killswitch)
{
  decoder_t *dec=(decoder_t *)handle;
  samples_t *s=&dec->a.s;
  int size;

  WaitForSingleObject(s->hMutex,INFINITE);

  while (STATE_RUN==s->state&&s->wp<s->mp) {
    SignalObjectAndWait(s->hMutex,s->hEventRead,INFINITE,FALSE);
    WaitForSingleObject(s->hMutex,INFINITE);
  }

  if (len<(size=s->wp-s->rep))
    size=len;

  memcpy(dest,s->rep,size);
  s->wp=s->rep;

  // signal the writer thread.
  SetEvent(s->hEventWrite);
  ReleaseMutex(s->hMutex);

  return size;
}

/*
 * Seek in the file.
 * parameters:
 * intptr_t handle: the handle you returned from the call to winampGetExtendedRead_open
 * int millisecs: the number of milliseconds into the file (measured starting from the beginning of the file) that you should seek to.
 * return value: nonzero on success, zero on failure.
 * 
 * note: this is optional, and isn't used by anything in the wild at the moment. But it will be soon.
 * Your seeking doesn't have to be totally accurate.
 */
/* return nonzero on success, zero on failure. */
__declspec( dllexport ) int winampGetExtendedRead_setTime(intptr_t handle, int millisecs)
{
  return 0;
}

/*
 * close the file and free all resources associated with it.
 * parameters:
 * intptr_t handle: the handle you returned from the call to winampGetExtendedRead_open
 * no return value.
 * 
 * Notes:
 * If you're coding in C++, remember to wrap these function definitions in extern "C" { ... };, so they are exported properly.
 * 
 * If you implement this API then it will be used by the CD burner (as opposed to your implementation of in2.h)
 */
__declspec( dllexport ) void winampGetExtendedRead_close(intptr_t handle)
{
  decoder_t *dec=(decoder_t *)handle;

  decoder_kill(dec);
  decoder_wait(dec);
  decoder_stop(dec,NULL,NULL);
  decoder_cleanup(dec);
  free(dec);
}
#endif // }
