/*
 * prefdef.h
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#ifndef PREFDEF_H
#define PREFDEF_H

#include "resources.h"
#include "prefs.h"

#define ARRAY_SIZE(x) \
  ((sizeof (x))/(sizeof (x)[0]))

///////////////////////////////////////////////////////////////////////////////
#define SLIDERI(imin,imax) \
  (slider_t){ .i.min=imin,.i.max=imax }
#define SLIDERX(xmin,xmax,xstep) \
  (slider_t){ .x.min=xmin,.x.max=xmax,.x.step=xstep }

///////////////////////////////////////////////////////////////////////////////
#define PLAY_GENERAL "play-general"
#define CONV_GENERAL "conv-general"

#define SIZEOF(T,f) \
  (sizeof (((T *)NULL)->f))

#define MKSTR(id) \
  #id
#define PREFDEF(play,group,name) \
  MKSTR(play-group),#name,offsetof(prefs_t,group.name),\
      SIZEOF(prefs_t,group.name)

#define PREFS_INT(play,group,name) \
  PREFDEF(play,group,name),PREF_INT
#define SLIDER_INT(play,group,name,id,min,max) \
  PREFS_INT(play,group,name),id,prefs_slider,.slider=&SLIDERI(min,max)

#define PREFS_DOUBLE(play,group,name) \
  PREFDEF(play,group,name),PREF_DOUBLE
#define SLIDER_DOUBLE(play,group,name,id,min,max,step) \
  PREFS_DOUBLE(play,group,name),id,prefs_slider,.slider=&SLIDERX(min,max,step)
 
#define PREFS_STRING(play,group,name) \
  PREFDEF(play,group,name),PREF_STRING

///////////////////////////////////////////////////////////////////////////////
#define GENERAL_PDS() (prefdef_t []){ \
  /* general */ \
  { PREFS_INT(play,general,bps),IDC_COMBO_WINAMP_BPS, \
      prefs_combo,.combo=(combo_t []){ \
        { "16",.i=16 }, \
        { "24",.i=24 }, \
        { NULL } \
      }}, \
  { PREFS_INT(play,general,vid),IDC_CHECKBOX_WINAMP_VID,prefs_check }, \
  { PREFS_INT(play,general,vis),IDC_CHECKBOX_WINAMP_VIS,prefs_check }, \
  /* rg */ \
  { PREFS_INT(play,rg,off),IDC_CHECKBOX_RG_OFF,prefs_check }, \
  { SLIDER_DOUBLE(play,rg,weight,IDC_SLIDER_RG_WEIGHT,0.0,1.0,0.01) }, \
  { SLIDER_DOUBLE(play,rg,preamp,IDC_SLIDER_RG_PREAMP,-12.0,12.0,0.1) }, \
  { PREFS_INT(play,rg,comment),IDC_CHECKBOX_RG_COMMENT,prefs_check }, \
}

enum { PREFS_GENERAL_N=ARRAY_SIZE(GENERAL_PDS()) };

typedef struct prefs_general {
  prefs_t *ps;
  prefgui_t pg[PREFS_GENERAL_N];

  struct {
    prefgui_t *weight;
    prefgui_t *preamp;
  } rg;
} prefs_general_t;

///////////////////////////////////////////////////////////////////////////////
#define CONV_GENERAL_PDS() (prefdef_t []){ \
  /* general */ \
  { PREFS_INT(conv,general,bps),IDC_COMBO_WINAMP_BPS, \
      prefs_combo,.combo=(combo_t []){ \
        { "16",.i=16 }, \
        { "24",.i=24 }, \
        { "32",.i=32 }, \
        { NULL } \
      }}, \
  /* rg */ \
  { PREFS_INT(conv,rg,off),IDC_CHECKBOX_RG_OFF,prefs_check }, \
  { SLIDER_DOUBLE(conv,rg,weight,IDC_SLIDER_RG_WEIGHT,0.0,1.0,0.01) }, \
  { SLIDER_DOUBLE(conv,rg,preamp,IDC_SLIDER_RG_PREAMP,-12.0,12.0,0.1) }, \
  { PREFS_INT(conv,rg,comment),IDC_CHECKBOX_RG_COMMENT,prefs_check }, \
}

enum { PREFS_CONV_GENERAL_N=ARRAY_SIZE(CONV_GENERAL_PDS()) };

typedef struct prefs_conv_general {
  prefs_t *ps;
  prefgui_t pg[PREFS_CONV_GENERAL_N];

  struct {
    prefgui_t *weight;
    prefgui_t *preamp;
  } rg;
} prefs_conv_general_t;

///////////////////////////////////////////////////////////////////////////////
#define AUDIO_PDS(play) (prefdef_t []){ \
  /* audio */ \
  { SLIDER_INT(play,audio,queue_size,IDC_SLIDER_AUDIO_QUEUE_SIZE,0,15) }, \
  /* sox */ \
  { PREFS_STRING(play,sox,mode),IDC_COMBO_SOX_MODE, \
      prefs_combo,.combo=(combo_t []){ \
        { "Off",.s="off" }, \
        { "Automatic",.s="auto" }, \
        { "Force",.s="on" }, \
        { NULL } \
      }}, \
  { PREFS_INT(play,sox,rate),IDC_COMBO_SOX_RATE, \
      prefs_combo,.combo=(combo_t []){ \
        { "Off",.i=0 }, \
        { "44100 Hz",.i=44100 }, \
        { "48000 Hz",.i=48000 }, \
        { "88200 Hz",.i=88200 }, \
        { "96000 Hz",.i=96000 }, \
        { "176400 Hz",.i=176400 }, \
        { "192000 Hz",.i=192000 }, \
        { NULL } \
      }}, \
  { PREFS_STRING(play,sox,rate_q),IDC_COMBO_SOX_RATE_Q, \
      prefs_combo,.combo=(combo_t []){ \
        { "Default",.s="" }, \
        { "Quick",.s="-q" }, \
        { "Low",.s="-l" }, \
        { "Medium",.s="-m" }, \
        { "High",.s="-h" }, \
        { "Very High",.s="-v" }, \
        { NULL } \
      }}, \
  { PREFS_INT(play,sox,rate_min),IDC_CHECKBOX_SOX_RATE_MIN,prefs_check }, \
  { PREFS_INT(play,sox,rate_mul),IDC_CHECKBOX_SOX_RATE_MUL,prefs_check }, \
  { PREFS_STRING(play,sox,dither),IDC_COMBO_SOX_DITHER, \
      prefs_combo,.combo=(combo_t []){ \
        { "Off",.s="off" }, \
        { "Default",.s="" }, \
        { "Sloped TPDF",.s="-S" }, \
        { "Noise Shaping",.s="-s" }, \
        { "Lipshitz Filter",.s="-f lipshitz" }, \
        { "F-Weighted Filter",.s="-f f-weighted" }, \
        { "Modified E-Weighted Filter",.s="-f modified-e-weighted" }, \
        { "Improved E-Weighted Filter",.s="-f improved-e-weighted" }, \
        { "Gesemann Filter",.s="-f gesemann" }, \
        { "Shibata Filter",.s="-f shibata" }, \
        { "Low Shibata Filter",.s="-f low-shibata" }, \
        { "High Shibata Filter",.s="-f high-shibata" }, \
        { NULL } \
      }}, \
}

///////////////////////////////////////////////////////////////////////////////
#define PLAY_AUDIO_PDS() AUDIO_PDS(play)

enum { PREFS_AUDIO_N=ARRAY_SIZE(PLAY_AUDIO_PDS()) };

typedef struct prefs_audio {
  prefs_t *ps;
  HBRUSH hbrBackground;
  prefgui_t pg[PREFS_AUDIO_N];

  struct {
    prefgui_t *queue_size;
  } audio;
} prefs_audio_t;

///////////////////////////////////////////////////////////////////////////////
#define CONV_AUDIO_PDS() AUDIO_PDS(conv)

enum { PREFS_CONV_AUDIO_N=ARRAY_SIZE(CONV_AUDIO_PDS()) };

typedef struct prefs_conv_audio {
  prefs_t *ps;
  HBRUSH hbrBackground;
  prefgui_t pg[PREFS_CONV_AUDIO_N];

  struct {
    prefgui_t *queue_size;
  } audio;
} prefs_conv_audio_t;

///////////////////////////////////////////////////////////////////////////////
typedef struct prefs_conv_page {
  prefs_conv_general_t general;
  prefs_conv_audio_t audio;
} prefs_conv_page_t;

#define PREFS_CONV_TCP() (tcp_t[]){ \
  { L"General",IDD_PREFS_CONV_GENERAL,ConvGeneralDlgProc, \
      offsetof(prefs_conv_page_t,general) }, \
  { L"Audio",IDD_PREFS_CONV_AUDIO,ConvAudioDlgProc, \
      offsetof(prefs_conv_page_t,audio) }, \
  { NULL,0,NULL } \
}

enum { PREFS_CONV_NPAGES=ARRAY_SIZE(PREFS_CONV_TCP())-1 };

typedef struct prefs_conv {
  prefs_t *ps;
  prefs_conv_page_t page;
  tabs_t tc;
  HWND hPages[PREFS_CONV_NPAGES];
} prefs_conv_t;

///////////////////////////////////////////////////////////////////////////////
#define VIDEO_PDS() (prefdef_t []){ \
  /* video */ \
  { SLIDER_INT(play,video,queue_size,IDC_SLIDER_VIDEO_QUEUE_SIZE,0,15) }, \
  { SLIDER_DOUBLE(play,video,min_drift,IDC_SLIDER_VIDEO_MIN_DRIFT, \
      -2000.0,0.0,10.0) }, \
  { SLIDER_DOUBLE(play,video,max_drift,IDC_SLIDER_VIDEO_MAX_DRIFT, \
      0.0,16000.0,10.0) }, \
}

enum { PREFS_VIDEO_N=(sizeof VIDEO_PDS())/(sizeof VIDEO_PDS()[0]) };

typedef struct prefs_video {
  prefs_t *ps;
  prefgui_t pg[PREFS_VIDEO_N];

  struct {
    prefgui_t *queue_size;
    prefgui_t *min_drift;
    prefgui_t *max_drift;
  } video;
} prefs_video_t;

#endif // PREFDEF_H
