/*
 * prefs_audio.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "prefdef.h"

#define PREFS_AUDIO

///////////////////////////////////////////////////////////////////////////////
typedef prefs_audio_t priv_t;

static prefdef_t pds[]=PLAY_AUDIO_PDS();
static prefdef_t *mpd=pds+PREFS_AUDIO_N;

///////////////////////////////////////////////////////////////////////////////
void prefs_audio_load(prefs_t *ps)
{
  prefdef_t *pd;

  for (pd=pds;pd<mpd;++pd)
    prefs_read_play(ps,pd);
}

///////////////////////////////////////////////////////////////////////////////
#define PREFS_SYNC_SLIDER
#include "slider.h"

static void priv_init(priv_t *p, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd;
  prefgui_t *pg;

  for (pd=pds,pg=p->pg;pd<mpd;++pd,++pg) {
    if (NULL!=pd->init) {
      pd->init(pg,pd,ps,hDlg);

      switch (pg->id) {
#define PREFS_INIT_SLIDER
#include "slider.h"
      default:
        break;
      }
    }
  }
}

static void priv_save(priv_t *p, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd;
  prefgui_t *pg;

  for (pd=pds,pg=p->pg;pd<mpd;++pd,++pg) {
    if (NULL!=pg->vmt) {
      pg->vmt->get(pg,ps,hDlg);
      prefs_write(ps,pd);
    }
  }
}

static void priv_display(priv_t *p, prefs_t *ps, HWND hDlg)
{
  prefdef_t *pd;
  prefgui_t *pg;

  for (pd=pds,pg=p->pg;pd<mpd;++pd,++pg) {
    if (NULL!=pg->vmt)
      pg->vmt->set(pg,ps,hDlg);
  }
}

///////////////////////////////////////////////////////////////////////////////
static void EnableSox(HWND hDlg)
{
  int enabled=0<SendDlgItemMessage(hDlg,IDC_COMBO_SOX_MODE,CB_GETCURSEL,0,0);

  EnableWindow(GetDlgItem(hDlg,IDC_STATIC_SOX_RATE_LABEL),enabled);
  EnableWindow(GetDlgItem(hDlg,IDC_COMBO_SOX_RATE),enabled);
  EnableWindow(GetDlgItem(hDlg,IDC_STATIC_SOX_DITHER_LABEL),enabled);
  EnableWindow(GetDlgItem(hDlg,IDC_COMBO_SOX_DITHER),enabled);
}

static void EnableSoxRate(HWND hDlg)
{
  int enabled=0<SendDlgItemMessage(hDlg,IDC_COMBO_SOX_MODE,CB_GETCURSEL,0,0)
      &&0<SendDlgItemMessage(hDlg,IDC_COMBO_SOX_RATE,CB_GETCURSEL,0,0);

  EnableWindow(GetDlgItem(hDlg,IDC_COMBO_SOX_RATE_Q),enabled);
  EnableWindow(GetDlgItem(hDlg,IDC_CHECKBOX_SOX_RATE_MIN),enabled);
  EnableWindow(GetDlgItem(hDlg,IDC_CHECKBOX_SOX_RATE_MUL),enabled);
}

static void Enable(HWND hDlg)
{
  EnableSox(hDlg);
  EnableSoxRate(hDlg);
}

///////////////////////////////////////////////////////////////////////////////
BOOL CALLBACK AudioDlgProc(HWND hDlg, UINT uMsg, WPARAM wParam,
    LPARAM lParam)
{
  priv_t *p=WM_INITDIALOG==uMsg?(void *)lParam
      :(void *)GetWindowLongPtr(hDlg,GWLP_USERDATA);

  switch(uMsg) {
  case WM_INITDIALOG:
	SetWindowLongPtr(hDlg,GWLP_USERDATA,lParam);
    priv_init(p,p->ps,hDlg);
    Enable(hDlg);
    return TRUE;
#if 0 // {
  // http://www.winprog.org/tutorial/dlgfaq.html
  case WM_CTLCOLORDLG:
	return (BOOL)CreateSolidBrush(RGB(255,255,255));
#endif // }
  case WM_COMMAND:
    switch(LOWORD(wParam)) {
    case IDOK:
      priv_save(p,p->ps,hDlg);
      return TRUE;
    case IDC_COMBO_SOX_MODE:
	  Enable(hDlg);
      return TRUE;
    case IDC_COMBO_SOX_RATE:
	  EnableSoxRate(hDlg);
      return TRUE;
    case IDC_BUTTON_DEFAULT:
      priv_display(p,(prefs_t *)lParam,hDlg);
	  Enable(hDlg);
      return TRUE;
    default:
	  break;
    }

    break;
  case WM_NOTIFY:
    switch (LOWORD(wParam)) {
#define PREFS_NOTIFY_SLIDER
#include "slider.h"
    default:
      break;
    }

    break;
  default:
    break;
  }

  return FALSE;
}
