/*
 * sox_writer.c
 * Copyright (C) 2010 - 2013 Peter Belkner <pbelkner@snafu.de>
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
#include "plugin.h"

///////////////////////////////////////////////////////////////////////////////
typedef struct priv {
  audio_t *a;
} priv_t;

static int priv_getopts(sox_effect_t *effp, int argc, char *argv[])
{
  priv_t *priv=effp->priv;
  int n=1;

  if (argc<2)
    return SOX_EOF;

  priv->a=(audio_t *)argv[n++];

  return SOX_SUCCESS;
}

#define PRIV_FLOW(bits) \
static int priv_flow_##bits(sox_effect_t *effp, sox_sample_t const *ibuf, \
    sox_sample_t *obuf, size_t *isamp, size_t *osamp) \
{ \
  enum { BYTES=(bits)>>3 }; \
  priv_t *priv=effp->priv; \
  samples_t *s=&priv->a->s; \
  const sox_sample_t *rp=ibuf; \
  const sox_sample_t *mrp=ibuf+*isamp; \
  int32_t sample; \
  char *wp,*mwp; \
  SOX_SAMPLE_LOCALS; \
 \
  while (STATE_RUN==s->state&&rp<mrp) { \
	samples_wait_lock(s); \
    wp=(char *)s->wp; \
    mwp=s->mp; \
 \
    while (/*STATE_KILL!=s->state&&*/rp<mrp&&wp<mwp) { \
      sample=SOX_SAMPLE_TO_SIGNED_##bits##BIT(*rp,effp->clips); \
	  ++rp; \
      memcpy(wp,&sample,BYTES); \
      wp+=BYTES; \
    } \
 \
    s->wp=wp; \
	samples_notify_unlock(s); \
  } \
 \
  *osamp=0; \
 \
  return mrp==rp?SOX_SUCCESS:SOX_EOF; \
}

PRIV_FLOW(16)
PRIV_FLOW(24)
PRIV_FLOW(32)

///////////////////////////////////////////////////////////////////////////////
static int priv_flow(sox_effect_t *effp, sox_sample_t const *ibuf,
    sox_sample_t *obuf, size_t *isamp, size_t *osamp)
{
  priv_t *priv=effp->priv;
  samples_t *s=&priv->a->s;

  switch (s->bps) {
  case 16:
    return priv_flow_16(effp,ibuf,obuf,isamp,osamp);
  case 24:
    return priv_flow_24(effp,ibuf,obuf,isamp,osamp);
  case 32:
    return priv_flow_32(effp,ibuf,obuf,isamp,osamp);
  default:
    return SOX_EOF;
  }
}

///////////////////////////////////////////////////////////////////////////////
sox_effect_handler_t const * ffsox_writer() {
  static sox_effect_handler_t handler={
    // Effect name
    .name="ffsox_writer",
    // Short explanation of parameters accepted by effect
    .usage=NULL,
    // Combination of SOX_EFF_* flags
    .flags=SOX_EFF_MCHAN,
    // Called to parse command-line arguments (called once per effect).
    .getopts=priv_getopts,
    // Called to initialize effect (called once per flow).
    .start=NULL,
    // Called to process samples.
    .flow=priv_flow,
    // Called to finish getting output after input is complete.
    .drain=NULL,
    // Called to shut down effect (called once per flow).
    .stop=NULL,
    // Called to shut down effect (called once per effect).
    .kill=NULL,
    // Size of private data SoX should pre-allocate for effect
    .priv_size=sizeof (priv_t)
  };

  return &handler;
}
